# -*- coding: utf-8 -*-
"""
Created on Sun May  6 13:45:47 2018

@author: Michael Stadlhofer 01530415
"""

import numpy as np
from mo_functions import atom, molecule, degree, list_txt

#bool variable to indicate if slaters rule for the effective nuclear charge
# is used
s = True

'''defining molecules'''
# H2+    
H1 = atom([[1,0,0]], 1, [-0.76e-10/2.0,0,0], symbol = "H1", slater=s)
H2 = atom([[1,0,0]], 1, [0.76e-10/2.0,0,0], symbol = "H2", slater=s)
H2plus = molecule([H1, H2], "H2+")


# H2O
alpha = 90 # angle between symmetry axis of the molecule in the x,y plane 
           # and the x axis
O = atom([[1,0,0],[2,0,0], [2,1,-1], [2,1,1], [2,1,0]], 8, [0,0,0], symbol = "O", slater=s)
H1 = atom([[1,0,0]], 1, 0.097e-9*np.asarray([np.cos((alpha+52)*degree), np.sin((alpha+52)*degree),0]), symbol = "H1", slater=s)
H2 = atom([[1,0,0]], 1, 0.097e-9*np.asarray([np.cos((alpha -52)*degree), np.sin((alpha - 52)*degree),0]), symbol = "H2", slater=s)
H2O = molecule([O, H1, H2], "H2O")


''' solving the roothan equations for a molecule'''
# enter the molecule to be solved here
mol = H2O

# solving the roothaan equations
mol.solve()

# solution to the eigenvalue problem
eig = mol.sol

# overlap and hamiltonian matrix
S = mol.S
print("Overlap matrix")
print(S)
print("\n")

H = mol.H
print("Hamiltonian matrix")
print(H)
print("\n")



# saving the calculated matrix elements
def save_matrix(name, m):
    with open(name + ".txt", "w") as op:
        for line in m:
            op.write(list_txt([str(_) for _ in line], ", "))
            op.write("\n")
 
    
Hrounded = np.round(H, 3)
Srounded = np.round(S, 3)
       
save_matrix("H_H2O_rounded", Hrounded)
save_matrix("S_H2O_rounded", Srounded)

save_matrix("H_H2O", H)
save_matrix("S_H2O", S)


# solution
print("eigenvalues E  and eigenvectors v \n")
for _ in eig:
    print(r" E" + str(eig.index(_)) + ": " + str(_[0]) + " eV")
    print(" v" + str(eig.index(_)) + ": ")
    for __ in _[1]:
        print("      " + str(__))
    print("\n")

''' saving the molecule (placement of atoms and orbitals)
and the solution to the roothaan equations'''
# if the roothaan equations have not been solved yet, this function saves the
# construction of the molecule to a txt file
mol.save_molecule_data()


''' testing '''
#import matplotlib.pyplot as plt
#relative error of the expectation value <psi|H|psi>
# for different upper integration limits rmax
#
#orb_idx = 1
#
#Atom= O
#e_rel = lambda r: (mol.braket(Atom.orbitals[orb_idx], Atom.orbitals[orb_idx], "H", rmax= r) - Atom.orbitals[orb_idx].E)/Atom.orbitals[orb_idx].E
#
#
#R = np.linspace(0.1*a0, 100*a0, 10)
#err = [e_rel(r) for r in R]
#
#
#plt.plot(R/a0, np.abs(np.asarray(err)))
