# -*- coding: utf-8 -*-
"""
Created on Fri May 11 12:42:27 2018

@author: Michael Stadlhofer 01530415
"""

import numpy as np
import matplotlib.pyplot as plt
from mo_functions import atom, molecule, a0, load_construction
from mo_functions import load_solution,  aoSum_xy, V_xy, orbital_dict, list_txt, mol_dim





''' importing data / defining coefficients '''

#bool variable to indicate if slaters rule for the effective nuclear charge
# is used
s = True

# name of the molecule to be 
name = "H2O"




''' creating molecule structure'''
atom_parameters = load_construction(name + "_construction.txt")
# creating the atom objects for the molecule
atoms = []
for p in atom_parameters:
    atoms.append(atom(p[0], p[1], p[2], p[3], slater=s))  
# creating molecule from atoms
mol = molecule( atoms, name)





#################################
''' custom molecular orbitals '''
#################################

# custom combinations of the atomic orbitals can be entered here
eigenvectors = [
#     [],
#     [],
#     [weight 1, weight 2, weight 3, weight 4, ...]
     ]
# the values in the eigenvectors are the weights for the orbitals in mol.orbitallist
# in order to get the order right run the following line:
print("The molecular orbitals are in this order: \n" + list_txt(mol.orbitalnames, "  "))


# the energy of the custom orbitals is unknown (used for plotting)
E = [" ??? "]*len(mol.orbitalnames)







''' importing solution to roothaan equations from a file '''
# if no custum eigenfunctions were given
if len(eigenvectors) == 0: 
    solution = load_solution(name + "_solution.txt")   
    
    # getting energy and eigenvectors from the imported solution data
    E = []
    eigenvectors = []
    for sol in solution:
           E.append(sol[0])
           eigenvectors.append(sol[1])





''' creating molecular orbital functions '''
# creating eigenfunctions (weighted sum of atomic orbitals)
#   using the eigenvectors 
eigenfunctions = []
for eigenvector in eigenvectors:
    eigenfunctions.append(aoSum_xy(mol.orbitallist, eigenvector))
    

# generating orbital names for the titles of the plots
orbitalnames = []
for j in range(len(mol.orbitallist)):
    orbitalnames.append(mol.atomlist[j].symbol + ":" + orbital_dict[mol.orbitallist[j].n, mol.orbitallist[j].l, mol.orbitallist[j].m])





''' plotting the molecular orbitals '''
# minimum distance between points 
step = a0/120

__ , dims = mol_dim(mol)
xmin, xmax, ymin, ymax, __,__ = dims

# creating the 2d grid
x = np.arange(xmin, xmax, step)
y = np.arange(ymin, ymax, step)
x, y = np.meshgrid(x,y)



## probability
for i in range(len(eigenvectors)):
    plt.figure("probability  |  E: :" + str(E[i])  + " eV")
    plt.title(list_txt(orbitalnames, "     ") + "\n" + list_txt([round(_, 3) for _ in eigenvectors[i]], "     "), y = 1.03 ) 
    plt.imshow(np.abs(eigenfunctions[i](x,y)*np.conjugate(eigenfunctions[i](x,y))), extent = (xmin/a0, xmax/a0, ymax/a0, ymin/a0))
    plt.colorbar()
    plt.xlabel(r"$x / a_0$")
    plt.ylabel(r"$y / a_0$")
    #plt.savefig("p_E" + str(E[i])  + "eV.png",dpi = 150)
    
## real part
#for i in range(len(eigenvectors)):
#    plt.figure(r"real part | E: " + str(E[i]) + " eV")
#    plt.title(list_txt(orbitalnames, "     ") + "\n" + list_txt([round(_, 3) for _ in eigenvectors[i]], "     "), y = 1.03 ) 
#    plt.imshow(np.real(eigenfunctions[i](x,y)),extent = (xmin/a0, xmax/a0, ymax/a0, ymin/a0))  
#    plt.colorbar()
#    plt.xlabel(r"$x / a_0$")
#    plt.ylabel(r"$y / a_0$")
#    #plt.savefig("re_E" + str(E[i])  + "eV.png",dpi = 150)
#    
# potential
plt.figure("log(potential) of " + name)
plt.imshow(np.log(V_xy(mol)(x,y)), extent = (xmin/a0, xmax/a0, ymax/a0, ymin/a0))
plt.xlabel(r"$x / a_0$")
plt.ylabel(r"$y / a_0$")
