# -*- coding: utf-8 -*-


# -*- coding: utf-8 -*-
"""
Created on Tue Mar 20 13:51:50 2018

@author: Michael Stadlhofer 01530415
"""

import numpy as np
from mayavi import mlab
from mo_functions import atom, molecule, a0, load_construction, load_solution, aoSum, mol_dim, list_txt





''' importing data / defining coefficients '''

#bool variable to indicate if slaters rule for the effective nuclear charge
# is used
s= True

# name of the molecule 
name = "H2O"

# index of eigenfunction to be plotted
# 0 is the eigenfunction with the lowest energy
ef_idx =6
# has to be lower than the number of eigenvectors




''' creating molecule structure'''
atom_parameters = load_construction(name + "_construction.txt")
# creating the atom objects for the molecule
atoms = []
for p in atom_parameters:
    atoms.append(atom(p[0], p[1], p[2], p[3],  slater=s))  
# creating molecule from atoms
mol = molecule( atoms, name)





#################################
''' custom molecular orbitals '''
#################################

# custom combinations of the atomic orbitals can be entered here
eigenvectors = [      
#     [],
#     [],
#     [0,1,3,0,1,2,2]
     ]
# the values in the eigenvectors are the weights for the orbitals in mol.orbitallist
# in order to get the order right run the following line:
print("The molecular orbitals are in this order: \n" + list_txt(mol.orbitalnames, "  "))





''' importing solution to roothaan equations from a file '''
# if no custum eigenfunctions were given
if len(eigenvectors) == 0: 
    solution = load_solution(name + "_solution.txt")   


    
    # getting energy and eigenvectors from the imported solution data
    E = []
    eigenvectors = []
    for sol in solution:
           E.append(sol[0])
           eigenvectors.append(sol[1])





''' creating molecular orbital functions '''
# creating eigenfunctions (weighted sum of atomic orbitals)
#   using the eigenvectors 
eigenfunctions = []
for eigenvector in eigenvectors:
    eigenfunctions.append(aoSum(mol.orbitallist, eigenvector))
    




''' plotting the molecular orbital '''
# minimum distance between points 
step = a0/25

# minimum and lmaximum values for the x, y and z directions
_ , dims = mol_dim(mol)
xmin, xmax, ymin, ymax, zmin, zmax = dims


# number of datapoints in each direction
xsteps = int((xmax - xmin)/step)*1j
ysteps = int((ymax - ymin)/step)*1j
zsteps = int((zmax - zmin)/step)*1j

# creating the 3d grid
x, y, z = np.pi*np.mgrid[xmin:xmax:xsteps, ymin:ymax:ysteps, zmin:zmax:zsteps]


# probability p of molecular orbital (orb)
def p(orb):
    return lambda x,y,z: np.real(np.conjugate(orb(x,y,z))*orb(x,y,z))
    
    
    
# probability density at each point of the grid
#src = p(eigenfunctions[ef_idx])(x,y,z)

# real part of the mo wavefunction at each point of the grid
src = np.real(eigenfunctions[ef_idx](x,y,z))

# range of the values of the wavefunction/ probabiliy density
datarange = src.max()-src.min()

# number of contours to be plotted
num_c = 16

# values at which contours are plotted
c = np.ndarray.tolist(np.linspace(-datarange, datarange, num_c))

#plotting the contours
mlab.contour3d(src,  contours = c,opacity = 0.2)


# plotting a contour for each atom
# (visualizing the positions of the atoms)
for i in range(len(mol.atoms)):
    
    o = mol.Vmo_xyz(x,y,z, i)
    datarange = o.max()-o.min()
  
    mlab.contour3d(o,  contours = [datarange*1.0/2.0],opacity = 1)

# showing the output
mlab.show()

